/* BalooHTTP
 * Author: Johan Ho
 */

using System;
using System.Net;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace Baloo
{
    /// <summary>
    /// This class fetches information from a URL. This class should later be extended
    /// to support fetching nontext replies from URLs. (In this project if there's
    /// enough time, or in a future project.)
    /// </summary>
    class BalooHTTP
    {
        /// <summary>
        /// Fetches a text reply from the specified URL, or null if an error occured.
        /// </summary>
        /// <param name="url">The URL to fetch a reply from.</param>
        /// <param name="parameters">Any eventual parameters to add to the url.</param>
        /// <returns>The text returned by the URL, or null if there's an error.</returns>
        public BalooReply FetchURL(string url, string parameters)
        {
            try
            {
                BalooReply reply = null;
                HttpWebRequest request;
                if (String.IsNullOrEmpty(parameters))
                    request = (HttpWebRequest)HttpWebRequest.Create(url);
                else request = (HttpWebRequest)HttpWebRequest.Create(url+"?parameters="+parameters);
                HttpWebResponse response = (HttpWebResponse) request.GetResponse();
                // If content-disposition is given with a filename, use that filename
                string filename=null;

                // First try to fetch filename from the content-disposition header
                string disposition = response.Headers.Get("Content-disposition");
                if (disposition != null)
                {
                    int filenamepos = disposition.IndexOf("filename=");
                    filename = disposition.Substring(filenamepos + "filename=".Length);
                }
                // Then try to fetch filename from url
                /*if (filename == null && !url.EndsWith('/'))
                {
                    Regex regex = new Regex("http://[0-9a-zA-Z-.]+(/[0-9a-zA-Z_%-~.]+)*");
                    Match match = regex.Match(url);
                    // The first group is the full match text, which would be the url.
                    // So it has to be more than one group for there to be any text matched in
                    // the parenthesis expression.
                    if (match.Groups.Count > 1)
                    {
                        Capture c = match.Groups[match.Groups.Count - 1].Captures[0];
                        if (c.Value.Contains("."))
                            filename = c.Value;
                    }
                }*/

                string contenttype = response.ContentType;
                string fileextension = null, contenttypeextension = null;
                // Remove charset from content-type
                int dividerpos = contenttype.LastIndexOf(';');
                if (dividerpos >= 0)
                    contenttype = contenttype.Substring(0, dividerpos);

                // Fetch file extension
                dividerpos = url.LastIndexOf('.');
                if (dividerpos >= 0 && dividerpos < url.Length - 2)
                    fileextension = url.Substring(dividerpos + 1); // m fjerne parameters

                // Fetch content-type "extension" (actually description)
                dividerpos = contenttype.IndexOf('/');
                if (dividerpos >= 0 && dividerpos < contenttype.Length - 2)
                    contenttypeextension = contenttype.Substring(dividerpos + 1);
                
                // If the url contains the filename (i.e. contains the path to the file
                // with the same extension as what's given in the content-type, then
                // fetch the filename from the url.
                if (fileextension.Equals(contenttypeextension) ||
                    (fileextension.Equals("txt") && contenttypeextension.Equals("plain")) || 
                    (fileextension.Equals("jpg") && contenttypeextension.Equals("jpeg")) ||
                    (fileextension.Equals("mp3") && contenttypeextension.Equals("mpeg")))
                {
                    dividerpos = url.LastIndexOf('/');
                    if (dividerpos <= url.Length-2)
                        filename = url.Substring(dividerpos+1);
                }

                // If no filename was fetched, try something else
                if (filename == null)
                {
                    // This isn't really how content-type should be used, but it should still
                    // work with most common file types...
                    if (contenttypeextension != null)
                    {
                        if (contenttypeextension.Equals("plain"))
                            filename = "Balooreply.txt";
                        else if (contenttypeextension.Equals("mpeg"))
                            filename = "Balooreply.mp3";
                        else filename = "Balooreply." + contenttypeextension;
                    }
                    // Since html is actually the most common when using url, assume html
                    // when no other file information has been found.
                    else filename = "Balooreply.html";
                }

                if (contenttype.StartsWith("text"))
                {
                    StreamReader reader;
                    if (response.ContentType.Contains("charset="))
                        reader = new StreamReader(response.GetResponseStream(), Encoding.GetEncoding(response.ContentType.Substring(response.ContentType.IndexOf("=")+1)));
                    else if (!String.IsNullOrEmpty(response.ContentEncoding))
                        reader = new StreamReader(response.GetResponseStream(),Encoding.GetEncoding(response.ContentEncoding));
                    else reader = new StreamReader(response.GetResponseStream());
                    string result = reader.ReadToEnd();
                    response.Close();
                    return new BalooReply(filename, result);
                }
                else
                {
                    Stream stream = response.GetResponseStream();
                    byte[] buffer = new byte[response.ContentLength];
                    int bytesread = 0;
                    while (bytesread < response.ContentLength)
                        bytesread += stream.Read(buffer, bytesread, (int)(response.ContentLength - bytesread));
                    stream.Close();
                    response.Close();
                    return new BalooReply(filename, buffer);
                }
            }
            catch
            {
                return null;
            }
        }
    }
}
